
/*
 *  UnWindows 1.1.2
 *  change_colors.c
 *
 *  Richard L. Kline
 *  Rensselaer Polytechnic Institute
 *  16 May 1994
 *
 *  Minor revisions by Rich Burridge
 *  Sun Microsystems Inc.
 *  October 2001, October 2002.
 *
 *  This is the color selection interface to the coloreyes
 *  program.  It allows the user to choose the
 *  colors he or she wants to associate with the distance
 *  indication given by coloreyes.
 *  Any number of colors from one to ten can be chosen.
 *
 *  ---
 *  Modification history:
 *  1.1.3 - October 2002 - See ChangeLog file.
 *  1.1.1 - October 2001 - See ChangeLog file.
 *  1.1   - 16 May 94    - all colors are now private read/write cells,
 *                         preventing the program from chewing up all
 *                         available colors on the display and allowing
 *                         the eyes to immediately reflect new settings
 */
    
#include <stdio.h>
#include <stdlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Cardinals.h>
#include <math.h>
#include <X11/Shell.h>
#include <X11/Xaw/Scrollbar.h>
#include <Xm/Protocols.h>
#include "Eyes.h"

char xeyes_specs_file[128];

static void fds(Widget);
static int read_data();
static void change_color(Widget, XtPointer, XEvent *, Boolean *);
static void add_color_proc(Widget, XtPointer, XtPointer);
static void delete_color_proc(Widget, XtPointer, XtPointer);
static void save_colors_proc(Widget, XtPointer, XtPointer);
static void realign_colors_proc(Widget, XtPointer, XtPointer);
static void done_proc(Widget, XtPointer, XtPointer);
static void rgb_scroll(Widget, XtPointer, XtPointer);

extern void WindowMessageCB(Widget, XtPointer, XtPointer);

static Display *dpy;
static Colormap cmap;

#define INIT_INTENS_RED    0.5
#define INIT_INTENS_GREEN  0.5
#define INIT_INTENS_BLUE   0.99
#define INIT_COLOR_STRING  "#8080ff"

#define INTENS(x)          ((int) ((x)*255))
#define RED                101
#define GREEN              102
#define BLUE               103
#define SPOTS              51.0
#define SCROLL_INC         (1.0 / SPOTS)

static Widget done;
static Widget add_color;
static Widget delete_color;
static Widget save_colors;
static Widget realign_colors;

static Widget red_scroll, green_scroll, blue_scroll;
static Widget red_label, green_label, blue_label;
static Widget color_box;

extern Atom XA_WM_DELETE_WINDOW;
extern EyesColorRec dis_colors[];     /* variable number of color-% pairs */
extern int num_colors;
static Boolean delete_pane = False;

static Pixel rwpixel;                 /* color for slider bars' output */
Pixel boxpix[10];                     /* colors for eyes and colorbox */
static char rwcolor[20] = INIT_COLOR_STRING;

Widget settingsWindow = NULL;

static Widget pane;
static Widget main_box;

static int pane_ctr = 0;
static int pane_num[10];


Pixel
set_color(Display *dpy, Colormap cmap, XColor *xcolor)
{
    if (!XAllocColor(dpy, cmap, xcolor)) {
        FPRINTF(stderr, "coloreyes: can't allocate color:\n");
        FPRINTF(stderr, "All colorcells allocated and\n");
        FPRINTF(stderr, "no matching cell found.\n");
        exit(1);
    }

    return(xcolor->pixel);
}


void
init_settings(Display *parent_dpy, Colormap parent_cmap, Widget parent)
{
    char *home;
    XColor xcolor;

    dpy = parent_dpy;
    cmap = parent_cmap;

    if ((home = getenv("HOME")) != NULL) {
        SPRINTF(xeyes_specs_file, "%s/.coloreyes", home);
    } else {
        STRCPY(xeyes_specs_file, "/.coloreyes");
    }

    /* A box to hold everything */    
    settingsWindow = XtVaCreatePopupShell("settings",
                                          transientShellWidgetClass, parent,
                                          XtNheight, 490, 
                                          XtNwidth, 830,
                                          XtNtitle, "Coloreyes Settings", 
                                          NULL);
    main_box = XtVaCreateManagedWidget("main_box", 
                                       compositeWidgetClass, settingsWindow,
                                       XtNheight, 490, 
                                       XtNwidth, 830, 
                                       NULL);

    /* The REALIGN COLORS button */
    realign_colors = XtVaCreateManagedWidget("Realign Colors",
                                             commandWidgetClass, main_box, 
                                             XtNx, 5, 
                                             XtNy, 30, 
                                             NULL);
    XtAddCallback(realign_colors, XtNcallback, realign_colors_proc, NULL);

    /* The DELETE COLOR button */
    delete_color = XtVaCreateManagedWidget("Delete Color", 
                                           commandWidgetClass, main_box, 
                                           XtNx, 5, 
                                           XtNy, 100, 
                                           NULL);
    XtAddCallback(delete_color, XtNcallback, delete_color_proc, NULL);

    /* The ADD COLOR button */
    add_color = XtVaCreateManagedWidget("Add Color", 
                                        commandWidgetClass, main_box, 
                                        XtNx, 5, 
                                        XtNy, 170, 
                                        NULL);
    XtAddCallback(add_color, XtNcallback, add_color_proc, NULL);

    /* The SAVE COLORS button */
    save_colors = XtVaCreateManagedWidget("Save settings", 
                                          commandWidgetClass, main_box, 
                                          XtNx, 5, 
                                          XtNy, 430, 
                                          NULL);
    XtAddCallback(save_colors, XtNcallback, save_colors_proc, NULL);

    /* The DONE button */
    done = XtVaCreateManagedWidget("Done", 
                                   commandWidgetClass, main_box,
                                   XtNx, 730, 
                                   XtNy, 430, 
                                   NULL);
    XtAddCallback(done, XtNcallback, done_proc, NULL);

    /* A box to show current color selection */
    color_box = XtVaCreateManagedWidget("color_box", 
                                        compositeWidgetClass, main_box, 
                                        XtNx, 5, 
                                        XtNy, 250,
                                        XtNwidth, 150, 
                                        XtNheight, 150, 
                                        NULL);

    /* A box of colors that the eyes become */
    pane = XtVaCreateManagedWidget("Paned", 
                                   panedWidgetClass, main_box,
                                   XtNx, 320, 
                                   XtNy, 10, 
                                   XtNwidth, 500,
                                   XtNheight, 200, 
                                   XtNorientation, XtorientHorizontal, 
                                   NULL);

    /* Store initial color for slider bar box. */
    XParseColor(dpy, cmap, INIT_COLOR_STRING, &xcolor);
    rwpixel = set_color(dpy, cmap, &xcolor);
    fds(main_box);

    num_colors = read_data();

    XmAddWMProtocolCallback(settingsWindow, XA_WM_DELETE_WINDOW,
                            WindowMessageCB, (caddr_t) 0);
}


/* Converts a color's name to its value. */

static int
ConvertColor(Widget w, char *color_name)
{
    XrmValue from, to;

    from.size = strlen(color_name) + 1;
    from.addr = color_name;

/* This conversion accepts a colorname from rgb.txt, or a #rrrgggbbb
 * rgb color definition, or the special toolkit strings "XtDefaultForeground"
 * and "XtDefaultBackground".
 */

    XtConvert(w, XtRString, (XrmValuePtr) &from, XtRPixel, (XrmValuePtr) &to);
    if (to.addr == NULL) {
        return(-1);
    }

    return((int) *((Pixel *) to.addr));
}


/* This procedure is called when the ADD_COLOR button is pressed.
 * It adds another panel to the pane widget.  The color of the panel 
 * is the currently selected color.
 */

/*ARGSUSED*/
static void
add_color_proc(Widget widget, XtPointer closure, XtPointer call_data)
{
    Arg args[10], gargs[5];
    int pane_width, box_width, cnt, i, numchildren, numpanes;
    Widget box;
    XColor xcolor;
    WidgetList children;

    if (num_colors == 10) {
        FPRINTF(stderr, "coloreyes: you cannot add another color\n");    
        return;
    }

    XtSetArg(gargs[0], XtNheight, 0);
    XtSetArg(gargs[1], XtNwidth, 0);
    XtGetValues(pane, gargs, 2);
    pane_width = (int) (gargs[1].value);
    if (num_colors == 0) {
        box_width = pane_width;
    } else {
        box_width = pane_width / num_colors;
    }

    cnt = 0;

    XParseColor(dpy, cmap, rwcolor, &xcolor);
    boxpix[num_colors] = set_color(dpy, cmap, &xcolor);
    XtSetArg(args[cnt], XtNbackground, boxpix[num_colors]); 
    cnt++;
    XtSetArg(args[cnt], XtNpreferredPaneSize, box_width);
    cnt++;

    box = XtCreateManagedWidget("box", boxWidgetClass, pane, args, cnt);
    XtAddEventHandler(box, ButtonPressMask, FALSE, change_color,
                      (XtPointer) pane_ctr);
    pane_num[num_colors] = pane_ctr++;

    XtSetArg(gargs[0], XtNnumChildren, &numchildren);
    XtSetArg(gargs[1], XtNchildren, &children);
    XtGetValues(pane, gargs, 2);
  
    numpanes = XawPanedGetNumSub(pane);
    box_width = pane_width / numpanes;

    XawPanedSetRefigureMode(pane, False);
    for (i =0 ; i < numchildren; i++) {
        if (strcmp(XtName(children[i]), "box") == 0) {
            cnt = 0;
            XtSetArg(args[cnt], XtNpreferredPaneSize, box_width);
            cnt++;
            XtSetArg(args[cnt], XtNwidth, box_width); 
            cnt++;
            XtSetValues(children[i], args, cnt);
        }
    }

    XawPanedSetRefigureMode(pane, True);
    XtManageChildren(children, numchildren);

    num_colors++;
}


/* Called when the REALIGN COLORS button is pressed
 *
 *  - this procedure divides the real estate equally among the colors.
 *
 *  - For example, if there are two colors, red and blue, and red has 75% 
 *    of the real estate and blue the other 25%, pressing REALIGN COLORS 
 *    will give each 50%.
 */

/*ARGSUSED*/
static void
realign_colors_proc(Widget widget, XtPointer closure, XtPointer call_data)
{
    Arg args[10], gargs[5];
    int pane_width, box_width, cnt, i, numchildren, numpanes;
    WidgetList children;
  
    if (num_colors == 10) {
        FPRINTF(stderr, "You cannot add another color\n");
        return;
    }

    if (num_colors < 2) {
        return;
    }

    XtSetArg(gargs[0], XtNheight, 0);
    XtSetArg(gargs[1], XtNwidth, 0);
    XtGetValues(pane, gargs, 2);
    pane_width = (int) (gargs[1].value);
  
    if (XawPanedGetNumSub(pane) == 0) {
        box_width = pane_width;
    } else {
        box_width = pane_width / XawPanedGetNumSub(pane);
    }
  
    XtSetArg(gargs[0], XtNnumChildren, &numchildren);
    XtSetArg(gargs[1], XtNchildren, &children);
    XtGetValues(pane, gargs, 2);
  
    numpanes = XawPanedGetNumSub(pane);
    box_width = pane_width / numpanes;
  
    XawPanedSetRefigureMode(pane, False);
    for (i = 0; i < numchildren; i++) {
        if (strcmp(XtName(children[i]), "box") == 0) {
            cnt = 0;
            XtSetArg(args[cnt], XtNpreferredPaneSize, box_width);
            cnt++;
            XtSetArg(args[cnt], XtNwidth, box_width); 
            cnt++;
            XtSetValues(children[i],args,cnt);
        }
    }

    XawPanedSetRefigureMode(pane, True);
    XtManageChildren(children, numchildren);
}


/* This is called when the DELETE button is pressed. The boolean
 * delete_pane is set to True.  Subsequent clicking in a pane will
 * check this variable and delete the pane if it is set.  Otherwise,
 * the color of the panel will be changed to the currently selected 
 * color. See the change_color procedure.
 *
 * Pressing the button when delete_pane is True removes the button 
 * highlight and sets delte_pane to False, effectively cancelling 
 * the delete function.
 */

/*ARGSUSED*/
static void
delete_color_proc(Widget widget, XtPointer closure, XtPointer call_data)
{
    Arg args[10];
    String selected_color = "gray71";
    int pixel, i;

    if (delete_pane == False && num_colors > 1) {
        if ((pixel = ConvertColor(delete_color, selected_color)) == -1) {
            FPRINTF(stderr, "Cannot convert color: %s\n", selected_color);
        } else {
            i = 0;
            XtSetArg(args[i],XtNbackground, (Pixel) pixel); 
            i++;
            XtSetValues(widget, args, i);
        }
        delete_pane = True;
    } else {
        XtVaGetValues(done, XtNbackground, &pixel, NULL);
        XtVaSetValues(widget, XtNbackground, pixel, NULL);
        delete_pane = False;
    }
}


/* This is called when the SAVE button is pressed. The data is written to 
 * the xeyes_specs file.
 */

/*ARGSUSED*/
static void
save_colors_proc(Widget widget, XtPointer closure, XtPointer xevent)
{
    Arg gargs[5];
    int pane_width, cnt, i, numchildren;
    WidgetList children;
    float offsets[10];
    int off_index = 0;
    char colorname[10][20];
    FILE *fp;

    XtSetArg(gargs[0], XtNheight, 0);
    XtSetArg(gargs[1], XtNwidth, 0);
    XtGetValues(pane, gargs, 2);
    pane_width = (int) (gargs[1].value);

    XtSetArg(gargs[0], XtNnumChildren, &numchildren);
    XtSetArg(gargs[1], XtNchildren, &children);
    XtGetValues(pane, gargs, 2);

    for (i = 0; i < numchildren; i++) {
        if (strcmp(XtName(children[i]), "box") == 0) {
            int x, red, green, blue;
            XColor xcolor;

            cnt = 0;
            XtSetArg(gargs[cnt], XtNbackground, 0);
            cnt++;
            XtSetArg(gargs[cnt], XtNx, 0);
            cnt++;
            XtSetArg(gargs[cnt], XtNy, 0);
            cnt++;
            XtSetArg(gargs[cnt], XtNwidth, 0);
            cnt++;
            XtGetValues(children[i], gargs, cnt);

            x = (int) (gargs[1].value);
            offsets[off_index] = (float) 1.0 * ((1.0 * x) / (1.0 * pane_width));
            off_index++;

            xcolor.pixel = (int) (gargs[0].value);
            XQueryColor(dpy, cmap, &xcolor);
            red = xcolor.red >> 8;
            green = xcolor.green >> 8;
            blue = xcolor.blue >> 8;

            SPRINTF(colorname[i], "rgb:%02x/%02x/%02x", red, green, blue);
        }
    }

    if ((fp = fopen(xeyes_specs_file, "w")) == NULL) {
        FPRINTF(stderr, "unable to open %s\n", xeyes_specs_file);
    } else {
        for (cnt = 0; cnt < off_index; cnt++) {
            if (cnt == 0) {
                FPRINTF(fp, "%4.2f\t%4.2f\t%d\t%s\n", offsets[cnt],
                        offsets[cnt+1], cnt+100, colorname[cnt]);
            } else {
                if (cnt < off_index-1) {
                    FPRINTF(fp, "%4.2f\t%4.2f\t%d\t%s\n", offsets[cnt]+0.01,
                            offsets[cnt+1], cnt+100, colorname[cnt]);
                } else {
                    FPRINTF(fp, "%4.2f\t%4.2f\t%d\t%s\n", offsets[cnt]+0.01,
                            1.0, cnt+100, colorname[cnt]);
                }
            }
        }
        FCLOSE(fp);
    }
}


/* called when the DONE button is pressed. */

/*ARGSUSED*/
static void
done_proc(Widget widget, XtPointer closure, XtPointer call_data)
{
    Arg gargs[5];
    int pane_width, cnt, i, numchildren;
    WidgetList children;
    float offset[10];

    XtSetArg(gargs[0],XtNheight,0);
    XtSetArg(gargs[1],XtNwidth,0);
    XtGetValues(pane,gargs,2);
    pane_width = (int)(gargs[1].value);

    XtSetArg(gargs[0],XtNnumChildren, &numchildren);
    XtSetArg(gargs[1],XtNchildren, &children);
    XtGetValues(pane,gargs,2);

    for (i = 0; i < numchildren; i++) {
        if (strcmp(XtName(children[i]), "box") == 0) {
            int x;

            cnt = 0;
            XtSetArg(gargs[cnt], XtNx, 0); 
            cnt++;
            XtGetValues(children[i],gargs,cnt);

            x = (int)(gargs[0].value);
            offset[i] = (float) 1.0 * ((1.0 * x) / (1.0 * pane_width));
        }
    }

    for (i = 1; i < num_colors; i++) {
        dis_colors[i-1].eyes_uoffset = offset[i] - 0.01;
        dis_colors[i].eyes_loffset   = offset[i];
    }
    dis_colors[num_colors].eyes_uoffset = 1.00;

    if (settingsWindow != NULL) {
        XtPopdown(settingsWindow);
    }
}


/*
 * This procedure is called when a button is pressed inside a panel. If the 
 * DELETE button has been previously pressed, the panel is deleted. If not,
 * the color of the panel is changed to that of the current color as 
 * specified by the scrollbar. The same color is also in the color box, and
 * its name is in the label widget.
 */

/*ARGSUSED*/
static void
change_color(Widget widget, XtPointer xtarg, XEvent *xevent, Boolean *bool)
{
    Arg arg[10];
    int i = 0;
    int pixel;
    int boxnum = (int) xtarg;
    XColor  xcolor;
    WidgetList pane_child;

    if (!num_colors) {
        return;
    }

    if (!delete_pane) {                              /* replace color */
        XParseColor(dpy, cmap, rwcolor, &xcolor);
        boxpix[boxnum] = set_color(dpy, cmap, &xcolor);
        i = 0;
        XtSetArg(arg[i], XtNbackground, xcolor.pixel); i++;
        XtSetValues(widget, arg, i);
    } else {                                         /* delete color */
        String color = "white";

        if ((pixel = ConvertColor(delete_color, color)) == -1) {
            FPRINTF(stderr, "Cannot convert color: %s\n", color);
        } else {
            i = 0;
            XtSetArg(arg[i], XtNbackground, (Pixel) pixel); 
            i++;
            XtSetValues(delete_color, arg, i);
        }

        XawPanedSetRefigureMode(pane, False);
        XtDestroyWidget(widget);
        delete_pane = False;
        num_colors--;

        /* Logically shift remaining color pane information to the left. */
        XtVaGetValues(pane, XtNchildren, &pane_child, NULL);
        i = 0;
        while (pane_num[i] != boxnum) {
            i++;
        }
        while (i < num_colors) {
            pane_num[i] = pane_num[i+1];
            xcolor.pixel = boxpix[i+1];
            XQueryColor(dpy, cmap, &xcolor);
            boxpix[i] = set_color(dpy, cmap, &xcolor);
            XtVaSetValues(pane_child[i], XtNbackground, boxpix[i-1], NULL);
            i++;
        }
        XawPanedSetRefigureMode(pane, True);
    }
}


static void
rgb_scroll(Widget w, XtPointer client_data, XtPointer position)
{
    static double rintensity = INIT_INTENS_RED, 
                  gintensity = INIT_INTENS_GREEN, 
                  bintensity = INIT_INTENS_BLUE;
    double intensity;
    int color;
    XColor xcolor;
    Widget w_label;
    char ch[20];

    /* Figure out which of the r, g, b scrollbars this is */
    if (w == red_scroll) {
        color = RED;
        intensity = rintensity;
        w_label = red_label;
    } else if (w == green_scroll) {
        color = GREEN;
        intensity = gintensity;
        w_label = green_label;
    } else {
        color = BLUE;
        intensity = bintensity;
        w_label = blue_label;
    }

    if ((int) client_data == 1) {           /* left or right button pressed */
        if ((int) position > 0) {           /* left button was pressed */
            if (intensity >= SCROLL_INC) {
                intensity = (int) (intensity * SPOTS + 0.001) / 
                            SPOTS - SCROLL_INC;
            } else {
                intensity = 0.0;
            }
        } else {                            /* right button was pressed */
            if (intensity < (1-SCROLL_INC)) {
                intensity = (int) (intensity * SPOTS + 0.001) / 
                            SPOTS + SCROLL_INC;
            } else {
                intensity = 1-SCROLL_INC;
            }
        }
    } else {                                /* middle button pressed */
        intensity = *((float *)position);
    }
    XawScrollbarSetThumb(w, intensity, SCROLL_INC);

    /* Update the appropriate color value and its label on the screen. */
    if (color == RED) {
        rintensity = intensity;
    } else if (color == GREEN) {
        gintensity = intensity;
    } else {
        bintensity = intensity;
    }
    SPRINTF(ch, "%d", (int) (intensity * 256));
    XtVaSetValues(w_label, XtNlabel, ch, NULL);

    SPRINTF(rwcolor, "rgb:%02x/%02x/%02x", 
                     (int) (rintensity * 255.9),
                     (int) (gintensity * 255.9), 
                     (int) (bintensity * 255.9));
    XParseColor(dpy, cmap, rwcolor, &xcolor);
    rwpixel = set_color(dpy, cmap, &xcolor);
    XtVaSetValues(color_box, XtNbackground, rwpixel, NULL);
}


/*
 * This procedure reads the xeyes_specs file to initialize the paned widget.
 * Each panel in the widget is assigned a color. The user can change the 
 * colors of panels, add new panels, or delete panels.
 *
 * Returns the number of colors read in.
 */

static int
read_data()
{
    char color[45], line[80], junk[35];     /* used in file read */
    float loffset, uoffset;                 /* used in file read */
    int number;                             /* used in file read */
    FILE *fp;                               /* used in file read */
    int i = 0;
    int cnt = 0;
    XColor xcolor;

    Arg args[10], gargs[5];
    int pane_width, box_width;
    Widget box;
    int defaults = 0;
    int done = 0;
  
    XtSetArg(gargs[0], XtNheight, 0);
    XtSetArg(gargs[1], XtNwidth, 0);
    XtGetValues(pane, gargs, 2);
    pane_width = (int) (gargs[1].value);

    if ((fp = fopen(xeyes_specs_file, "r")) == NULL) {
        /* FPRINTF(stderr, "unable to open %s\n", xeyes_specs_file); */
        defaults = 1;
    }

    while ( !done ) {
        if (!defaults) {
            if (fgets(line, 80, fp)) {
                SSCANF(line,"%f %f %d",&loffset, &uoffset, &number);
                STRCPY(junk, strtok(line," \t"));
                STRCPY(junk, strtok(NULL," \t"));
                STRCPY(junk, strtok(NULL," \t"));
                STRCPY(color, strtok(NULL,"\t\n"));
            } else {
                done = 1;
            }
        } else if (defaults == 1) {
            STRCPY(color, "red");
            loffset = 0;
            uoffset = 0.33;
            number = 100;
            defaults++;
        } else if (defaults == 2) {
            STRCPY(color, "green4");
            loffset = 0.34;
            uoffset = 0.66;
            number = 101;
            defaults++;
        } else if (defaults == 3) {
            STRCPY(color, "blue");
            loffset = 0.67;
            uoffset = 1;
            number = 102;
            done = 1;
        }

        if (defaults || !done) {
            box_width = pane_width * (uoffset - loffset);
            cnt = 0;

            XParseColor(dpy, cmap, color, &xcolor);
            boxpix[i] = set_color(dpy, cmap, &xcolor);
            XtSetArg(args[cnt], XtNbackground, boxpix[i]);
            cnt++;
            XtSetArg(args[cnt], XtNpreferredPaneSize, box_width);
            cnt++;

            /* make a box over each pane */
            box = XtCreateManagedWidget("box", boxWidgetClass, pane, args, cnt);
      
            /* When a button is pressed in the box, change the color of the 
             * pane to the color currently selected by the scrollbars and 
             * displayed in the box to their left.
             */
            XtAddEventHandler(box, ButtonPressMask, FALSE, change_color,
                              (XtPointer) pane_ctr);
            pane_num[i] = pane_ctr++;
            i++;
        }
    }

    if (!defaults) {
        FCLOSE(fp);
    }

    return(i);
}


/* This function automates the construction of a scrollbar. */    

static void
setscroll(parent, scroll_widget, name, scroll_proc, scroll_data, x, y)
    Widget *parent;
    Widget *scroll_widget;
    String name;
    void (*scroll_proc)(Widget, XtPointer, XtPointer);
    double scroll_data;
    int x, y;
{
    *scroll_widget = XtVaCreateManagedWidget(name,
                                           scrollbarWidgetClass, *parent,
                                           XtNx, x, 
                                           XtNy, y, 
                                           XtNwidth, 500, 
                                           XtNheight, 45,
                                           XtNorientation, XtorientHorizontal, 
                                           NULL);
    XawScrollbarSetThumb(*scroll_widget, scroll_data, SCROLL_INC);
    XtAddCallback(*scroll_widget, XtNscrollProc, (*scroll_proc), (XtPointer) 1);
    XtAddCallback(*scroll_widget, XtNjumpProc, (*scroll_proc), (XtPointer) 2);
}


/*  - load the font to display the colors' names
 *  - read the available colors from the system's color file
 *  - make a label widget the colors' names
 *  - make a scrollbar to riffle through the colors
 */

static void
fds(Widget box)
{
    XtVaSetValues(color_box, XtNbackground, rwpixel, NULL);

    XtVaCreateManagedWidget("red_letter",
                            labelWidgetClass, box,
                            XtNlabel, "R:",
                            XtNresize, False,
                            XtNborderWidth, 0, 
                            XtNx, 180, 
                            XtNy, 250, 
                            NULL);
    XtVaCreateManagedWidget("green_letter",
                            labelWidgetClass, box,
                            XtNlabel, "G:",
                            XtNresize, False,
                            XtNborderWidth, 0, 
                            XtNx, 180, 
                            XtNy, 302, 
                            NULL);
    XtVaCreateManagedWidget("blue_letter", 
                            labelWidgetClass, box,
                            XtNlabel, "B:", 
                            XtNresize, False,
                            XtNborderWidth, 0, 
                            XtNx, 180, 
                            XtNy, 354, 
                            NULL);
  
    red_label = XtVaCreateManagedWidget("red_label", 
                                        labelWidgetClass, box, 
                                        XtNlabel, "128", 
                                        XtNresize, False,
                                        XtNborderWidth, 0, 
                                        XtNx, 235, 
                                        XtNy, 250, 
                                        NULL);
    green_label = XtVaCreateManagedWidget("green_label", 
                                          labelWidgetClass, box,
                                          XtNlabel, "128", 
                                          XtNresize, False,
                                          XtNborderWidth, 0, 
                                          XtNx, 235, 
                                          XtNy, 302, 
                                          NULL);
    blue_label = XtVaCreateManagedWidget("blue_label", 
                                         labelWidgetClass, box,
                                         XtNlabel, "255", 
                                         XtNresize, False,
                                         XtNborderWidth, 0, 
                                         XtNx, 235, 
                                         XtNy, 354, 
                                         NULL);

    /* scroll bar to riffle through the colors */
    setscroll(&box, &red_scroll, "red_scroll", rgb_scroll,
              INIT_INTENS_RED, 320, 250);
    setscroll(&box, &green_scroll, "green_scroll", rgb_scroll, 
              INIT_INTENS_GREEN, 320, 302);
    setscroll(&box, &blue_scroll, "blue_scroll", rgb_scroll,
              INIT_INTENS_BLUE, 320, 354);
}
