
/*
 *  UnWindows 1.1.2
 *  xeyes.c
 *
 *  Gary Ormsby
 *  Rensselaer Polytechnic Institute
 *  Fall 1991
 *
 *  This program reads in the colors for the eyes and
 *  calls the main program routine.
 *
 *  Minor modifications by Richard L. Kline
 *  December 1993
 *
 *  Minor revisions by Rich Burridge
 *  Sun Microsystems Inc.
 *  October 2001, January 2002, October 2002.
 *
 *  Data is now read in from the file .coloreyes in the user's
 *  home directory instead of ~ormsbyg/proj/xeyes_specs.
 *  The eyes now respond to a pointer click by displaying the
 *  color editing window, which was formerly a separate program
 *  named change_colors.
 */

#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xlib.h>
#include <Xm/Protocols.h>
#include "Eyes.h"
#include <stdio.h> 
#include <stdlib.h>
#include "eyes.xbm"
#include "eyesmask.xbm"

/* Settings window control */
extern void init_settings();
extern Widget settingsWindow;

Atom XA_WM_DELETE_WINDOW;

static Widget toplevel;
static Display *dpy;
static Colormap cmap;

char xeyes_specs_file[128];

EyesColorRec dis_colors[10];        /* holds the colors */

int num_colors;
int max_x;
int max_y;

extern void exit();

String fallback_resources[] = {
    "*font:  -b&h-lucidatypewriter-bold-r-normal-sans-*-350-*-*-*-*-iso8859-*",
    "*Paned.x:                 310",
    "*Paned.y:                 10",
    "*Paned.width:             510",
    "*Paned.height:            200",
    "*Paned.Orientation:       horizontal",
    "*Paned*allowResize        True",
    "*Paned*resizeToPreferred: True",
    "*Paned*skipAdjust:        False",
    "*Paned*grip*height:       210",
    "*Paned*grip*width:        16",

    "*thickness:               25",
    "*red_scroll*x:            310",
    "*red_scroll*y:            250",
    "*green_scroll*x:          310",
    "*green_scroll*y:          302",
    "*blue_scroll*x:           310",
    "*blue_scroll*y:           354",
    "*Scrollbar*height:        45",
    "*Scrollbar*width:         510",
    "*Scrollbar*Orientation:   horizontal",

    "*appDefaultsVersion:      1",

    "XFontSel.cursor:          left_ptr",
    "*allowShellResize:        true",

    "*quitButton.Label:        quit",
    "*quitButton.left:         chainLeft",
    "*quitButton.right:        chainLeft",

    "*selectButton.Label:      select",
    "*selectButton.fromHoriz:  quitButton",
    "*selectButton.left:       chainLeft",
    "*selectButton.right:      chainLeft",
    
    NULL,
};


/* Used to respond to DELETE_WINDOW protocol request. */

void
WindowMessageCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    if (w == settingsWindow) {
        XtPopdown(settingsWindow);
        settingsWindow = NULL;
    } else {
        exit(0);
    }
}


/* Intrinsic error handler. */

static void
xeyesXtErrorHandler(String s)
{
    fprintf(stderr, "coloreyes: XToolkit Fatal Error\n%s\n", s);
    (void) abort();
}


/* Intrinsic warning handler. */

static void
xeyesXtWarningHandler(String s)
{
#ifdef DEBUG
    fprintf(stderr, "coloreyes: XToolkit Warning: %s\n", s);
#endif
}


/* Xlib error handler. */

static int 
xeyesXErrorHandler(Display *display, XErrorEvent *event)
{
    char msg[512];

    XGetErrorText(display, event->error_code, msg, 512);

    fprintf(stderr, "coloreyes: X Fatal Error %d on X Server %s\n%s\n", 
            event->error_code, DisplayString(display), msg);
        
    (void) abort();
        
    return(0);
}


/*
 *  This function reads the information in the xeyes_specs file. Its pathname 
 *  is in the xeyes_specs_file variable. Each record in the data file contains
 *  the information for one color. A record's format is:
 *
 *            starting %     ending %    id #    color
 *        --------------------------------------------------
 *  Type      float          float        int    String
 *
 *  A typical xeyes_specs file follows:
 *
 *      0.00    0.38    100    grey63
 *      0.39    0.47    101    SpringGreen1
 *      0.48    0.67    102    LightSalmon3
 *      0.68    1.00    103    IndianRed1
 *
 *  The first and second columns (starting % and ending %) delineate the 
 *  boundaries that the cursor must be between to change the eyes' to that 
 *  color. The distance is measured from the nearest edge of the screen to 
 *  the midpoint between the eyes' pupils.
 *
 *  Using the data above, if the cursor is between the left edge of the
 *  screen and the eyes, and the left screen edge is closest to the cursor,
 *  and the cursor is 53% of the way between the left edge and the
 *  pupils then the eyes will be LightSalmon3.  Moving the cursor
 *  further to the left slightly will cross the 48% boundary and change
 *  the color to SpringGreen1.  Likewise, moving the cursor to 
 *  the right (closer to the eyes) will cause the eyes to change to
 *  IndianRed1 when the 63% boundary is passed.
 *
 *  The colors can be visualized as bands encircling the eyes, like
 *  those on a dartboard.  The percentages indicate the thickness of
 *  of the bands.  With this analogy, the "last" color (in this case
 *  IndianRed1) is a circle whose center is the midpoint between the
 *  pupils.
 */

static int
read_dis_colors_rec()
{
    char line[80], junk[35];      /* used in file read */
    float loffset, uoffset;       /* used in file read */
    int number;                   /* used in file read */
    FILE *fp;                     /* used in file read */
    int i = 0;

    if ((fp = fopen(xeyes_specs_file, "r")) == NULL) {
        FPRINTF(stderr, "unable to open %s\n", xeyes_specs_file);
        FPRINTF(stderr, "using defaults\n");
        dis_colors[0].eyes_loffset = 0.0;
        dis_colors[0].eyes_uoffset = 0.32;
        dis_colors[0].number = 100;
        STRCPY(dis_colors[0].eyes_color, "red");
        dis_colors[1].eyes_loffset = 0.33;
        dis_colors[1].eyes_uoffset = 0.66;
        dis_colors[1].number = 101;
        STRCPY(dis_colors[1].eyes_color, "green4");
        dis_colors[2].eyes_loffset = 0.67;
        dis_colors[2].eyes_uoffset = 1.0;
        dis_colors[2].number = 102;
        STRCPY(dis_colors[2].eyes_color, "blue");
        i = 3;
    } else {
        while (fgets(line, 80, fp)) {
            SSCANF(line, "%f %f %d", &loffset, &uoffset, &number);
            dis_colors[i].eyes_loffset = loffset;
            dis_colors[i].eyes_uoffset = uoffset;
            dis_colors[i].number = number;
            STRCPY(junk, strtok(line," \t"));
            STRCPY(junk, strtok(NULL," \t"));
            STRCPY(junk, strtok(NULL," \t"));
            STRCPY(dis_colors[i].eyes_color, strtok(NULL,"\t\n"));
            i++;
        }
        FCLOSE(fp);
    }
    return(i);
}
    

/* Command line options table. Only resources are entered here. There is a
 * pass over the remaining options after XtParseCommand is let loose. 
 *
 * Exit with message describing command line format
 */

static void
usage()
{
    FPRINTF(stderr, "usage: xeyes\n");
    FPRINTF(stderr, "       [-geometry [{width}][x{height}]");
    FPRINTF(stderr, "[{+-}{xoff}[{+-}{yoff}]]] [-display [{host}]:[{vs}]]\n");
    FPRINTF(stderr, "       [-fg {color}] [-bg {color}] ");
    FPRINTF(stderr, "[-bd {color}] [-bw {pixels}]");
#ifdef SHAPE
    FPRINTF(stderr, " [-shape]");
#endif /*SHAPE*/
    FPRINTF(stderr, "\n");
    FPRINTF(stderr, "       [-outline {color}] [-center {color}] ");
    FPRINTF(stderr, "[-backing {backing-store}]\n");
    exit(1);
}


/* Verify that the colors read from the xeyes_specs_file
 *  are actual colors that can be accessed and rendered.
 */

static int
check_color(Display *dpy, Colormap cmap, 
            EyesColorRec dis_colors[], int num_colors)
{
    XColor junk;
    int i;

    for (i = 0; i < num_colors; i++) {
        if (XParseColor(dpy, cmap, dis_colors[i].eyes_color, &junk) == 0) {
            return(-1);
        } else {
            XAllocColor(dpy, cmap, &junk);
        }
    }
    return(0);
}


/*ARGSUSED*/
static void
change_colors(Widget w, XEvent *event, String *params, Cardinal *num)
{
    if (settingsWindow == NULL) {
        init_settings(dpy, cmap, toplevel);
    }
    XtPopup(settingsWindow, XtGrabNonexclusive);
}


/* These options were "borrowed" from MIT's xeyes. */

static XrmOptionDescRec options[] = {
    { "-outline", "*eyes.outline",      XrmoptionSepArg, NULL },
    { "-center",  "*eyes.center",       XrmoptionSepArg, NULL },
    { "-backing", "*eyes.backingStore", XrmoptionSepArg, NULL },
#ifdef SHAPE
    { "-shape",   "*eyes.shapeWindow",  XrmoptionNoArg,  "TRUE" },
#endif /*SHAPE*/
};


int
main(int argc, char *argv[])
{
    Widget eyes;
    Arg arg[2];
    char *home;
    char *labelname = NULL;
    int i;
    XtAppContext xtcontext;
    String trans = "<ButtonPress>:  change_colors()";
    static XtActionsRec window_actions[] = {
        { "change_colors", (void *) change_colors }
    };

    if ((home = getenv("HOME")) != NULL) {
        SPRINTF(xeyes_specs_file, "%s/.un_xeyes", home);
    } else {   
        STRCPY(xeyes_specs_file, "/.un_xeyes"); 
    } 

/* If this worked, we wouldn't need to have the separate app-defaults file 
 * Coloreyes.
 */
/*  toplevel = XtAppInitialize(&xtcontext, "Coloreyes",
 *                             options, XtNumber(options),
 *                             &argc, argv, fallback_resources, NULL, 0);
 */
    toplevel = XtInitialize("Coloreyes", "Coloreyes", options, 
                            XtNumber(options), &argc, argv);
    xtcontext = XtWidgetToApplicationContext(toplevel);

    /* Set the error handlers */
    XtAppSetErrorHandler(xtcontext, xeyesXtErrorHandler);
    XtAppSetWarningHandler(xtcontext, xeyesXtWarningHandler);
    (void) XSetErrorHandler(xeyesXErrorHandler);

    dpy = XtDisplay(toplevel);
    cmap = DefaultColormap(dpy, DefaultScreen(dpy));

    XA_WM_DELETE_WINDOW = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
    init_settings(dpy, cmap, toplevel);

    max_x = DisplayWidth(dpy, 0);
    max_y = DisplayHeight(dpy, 0);

    if (argc != 1) {
        usage();
    }

    i = 0;
    num_colors = read_dis_colors_rec();
    if (check_color(dpy, cmap, dis_colors, num_colors) == -1) {
        FPRINTF(stderr,"     Invalid color selected.\n");
        exit(1);
    }
    i = 0;
    XtSetArg(arg[i],
             XtNiconPixmap, XCreateBitmapFromData(XtDisplay(toplevel),
                                                  XtScreen(toplevel)->root,
                                                  (const char *) eyes_bits, 
                                                  eyes_width, eyes_height));
    i++;
    XtSetArg(arg[i], 
             XtNiconMask, XCreateBitmapFromData(XtDisplay(toplevel),
                                              XtScreen(toplevel)->root,
                                              (const char *) eyesmask_bits, 
                                              eyesmask_width, eyesmask_height));
    i++;
    XtSetValues(toplevel, arg, i);

    XtSetArg(arg[0], XtNlabel, &labelname);

    XtAppAddActions(xtcontext, window_actions, XtNumber(window_actions));

    eyes = XtVaCreateManagedWidget("eyes",
                                eyesWidgetClass, toplevel,
                                XtNtranslations, XtParseTranslationTable(trans),
                                NULL);
    XtGetValues(eyes, arg, 1);
    XtRealizeWidget(toplevel);

    XmAddWMProtocolCallback(toplevel, XA_WM_DELETE_WINDOW,
                            WindowMessageCB, (caddr_t) 0);

    XtAppMainLoop(xtcontext);

    return(0);
}
