
/*
 *  UnWindows 1.1.2
 *  win_sounds.c
 *
 *  This program is based on border.c by:
 *  Richard L. Kline
 *  Rensselaer Polytechnic Institute
 *  December 1993
 *
 *  Minor revisions by Rich Burridge
 *  Sun Microsystems Inc.
 *  October 2001, January 2002, October 2002.
 *
 *  This program monitors the mouse cursor's location and plays a sound
 *  if the cursor enters a window which has an associated entry in the
 *  user's ~/.un_sound file or whenever the cursor crosses an imaginary 
 *  threshold around any of the edges of the screen.
 *
 *  ---
 *  Modification history:
 *  1.1.3 - October 2002 - see the ChangeLog file.
 *  1.1.2 - January 2002 - see the ChangeLog file.
 *  1.1.1 - October 2001 - see the ChangeLog file.
 *  1.1   - 16 May 1994  - no changes
 */

#include <stdio.h>
#include <stdlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xatom.h>
#include <X11/Xmu/WinUtil.h>
#include "config.h"

#define FCLOSE  (void) fclose
#define FPRINTF (void) fprintf
#define SPRINTF (void) sprintf
#define STRCAT  (void) strcat
#define STRCPY  (void) strcpy

#define CHECK_INTERVAL 200    /* default time in ms between checks */
#define MAX_NAME_LEN   200L   /* truncate to this many */
#define THRESHOLD      10     /* width of active screen border */

#ifndef  AUDIO_PROG
#define  AUDIO_PROG      "/bin/audioplay"
#endif /*!AUDIO_PROG*/

static char *audio_prog;
static char audio_specs_file[128];
static char *ProgramName;              /* Name of program being executed */
static Display *dpy;
static Window last_child = 0;
static XtAppContext appCtx;
static int interval = CHECK_INTERVAL;  /* interval between pointer checks */
static int scr_width;                  /* width of the screen */
static int scr_height;                 /* height of the screen */

static void CheckPointer(XtPointer, XtSignalId *);
static void Usage();

extern int playit(char *, char **);


/*ARGSUSED*/
int
main(int argc, char **argv)
{
    char *home;
    int i;
    Widget toplevel;
  
    if ((home = getenv("HOME")) != NULL) {
        SPRINTF(audio_specs_file, "%s/.un_sound", home);
    } else {
        STRCPY(audio_specs_file, "/.un_sound");
    }
    audio_prog = strdup(AUDIO_PROG);

    ProgramName = argv[0];

    for (i = 1; i < argc; i++) {
        char *arg = argv[i];

        if (arg[0] == '-') {         /* process command line switches */
            switch (arg[1]) {
                case 'a':            /* -a <audio prog> */
                    if (++i >= argc) {
                        Usage();
                    }
                    audio_prog = strdup(argv[i]);
                    continue;

                case 'i':            /* -interval <milliseconds> */
                    if (++i >= argc) {
                        Usage();
                    }
                    interval = atoi(argv[i]);
                    continue;
                default:
                    Usage();
            }
        } else {                     /* any command line parameters */
            Usage();
        }
    }

    toplevel = XtInitialize("main", "Borders", NULL, 0, &argc, argv);

    dpy    = XtDisplay(toplevel);
    appCtx = XtWidgetToApplicationContext(toplevel);

    scr_width  = DisplayWidth(dpy, 0)  - THRESHOLD;
    scr_height = DisplayHeight(dpy, 0) - THRESHOLD;

    /* make up the first timeout */
    XtAppAddTimeOut(appCtx, interval, CheckPointer, NULL);

    /* off to the races */
    XtAppMainLoop(appCtx);

    return(0);
}


/*ARGSUSED*/
void
CheckPointer(XtPointer x, XtSignalId *signal)
{
    Window rep_root, child;
    int rep_rootx, rep_rooty;
    unsigned int rep_mask;
    int dx, dy;

    char *child_name = NULL;

    /*** VARIABLES FOR PLAY ***/
  
    char *play_argv[10];        /* arguments */
    char window[25];            /* used in file read */
    char audio_file[75];        /* used in file read */
    char volume[4];             /* used in file read */
    char *vol_str = "-v";       /* used in file read */
    FILE *fp;                   /* used in file read */
    char win_name[256];         /* set for a valid child window name or 
                                 * when a border is hit.
                                 */

    win_name[0] = '\0'; 

    XQueryPointer(dpy, RootWindow(dpy, DefaultScreen(dpy)), &rep_root,
                  &child, &rep_rootx, &rep_rooty, &dx, &dy, &rep_mask);

    if (child) {
        Window root;
        int dummyi;
        unsigned int dummy;

        if (XGetGeometry(dpy, child, &root, &dummyi, &dummyi,
                         &dummy, &dummy, &dummy, &dummy) && child != root) {
            child = XmuClientWindow(dpy, child);
        }

        if (child == last_child) {
            XtAppAddTimeOut(appCtx, interval, CheckPointer, NULL);
            return;
        } else {
            last_child = child;
        }

        if (!XFetchName(dpy, child, &child_name)) {
#ifdef DEBUG
            FPRINTF(stderr, "child window has no name.\n");
#endif /*DEBUG*/
        } else if (child_name) {
#ifdef DEBUG
            FPRINTF(stderr, "child window name: %s\n", child_name);
#endif /*DEBUG*/
            STRCPY(win_name, child_name);
            XFree(child_name);
        } else {
#ifdef DEBUG
            FPRINTF(stderr, "child window has no name.\n");
#endif /*DEBUG*/
        }
    }

    if (rep_rooty <= THRESHOLD) {
        STRCPY(win_name, "Top_Border");
    }
    if (rep_rooty >= scr_height) {
        STRCPY(win_name, "Bottom_Border");
    }
    if (rep_rootx <= THRESHOLD) {
        STRCPY(win_name, "Left_Border");
    }
    if (rep_rootx >= scr_width) {
        STRCPY(win_name, "Right_Border");
    }
  
    if (win_name[0] != '\0') {
        if ((fp = fopen(audio_specs_file, "r")) == NULL) 
            FPRINTF(stderr, "unable to open %s\n", audio_specs_file);
        else {
            while (fscanf(fp, "%s %s %s\n",
                   window, audio_file, volume) != EOF) {
                if (strcmp(window, win_name) == 0) {
                    if (strcmp(volume, "0") != 0) {
#ifdef DEBUG
                        FPRINTF(stderr, "Playing %s for child %s\n",
                                audio_file, win_name);
#endif /*DEBUG*/
                        play_argv[0] = audio_prog;
                        play_argv[1] = vol_str;
                        play_argv[2] = volume;
                        play_argv[3] = audio_file;
                        play_argv[4] = NULL;
                        playit(ProgramName, play_argv);
                    }
                }
            }
            FCLOSE(fp);
        }
    }

    /* Be sure to do this again in "interval" ms */
    XtAppAddTimeOut(appCtx, interval, CheckPointer, NULL);
}


/* Usage - Print out usage message and quit. */

static void
Usage()
{
    static char *help_message[] = {
        "where options include:",
        "  -a <audio program>  Alternate audio program to play sounds",
        "  -i <interval>    Interval in milliseconds between mouse pointer checks",
        "",
        NULL
    }, **cpp;

    FPRINTF(stderr, "Usage:\n        %s [-option ...]\n\n", ProgramName);
    for (cpp = help_message; *cpp; cpp++) {
        FPRINTF(stderr, "%s\n", *cpp);
    }

    exit(1);
}
